%   test.m 
%   pv = vector of parameters
%   plot = optional parameter; 1 if you want a plot
%   seed = optional parameter; specify a seed if you want one

clear;

pv = [3 3 3 10 3 3 2];
mcreps = 5000;
plot = 1;
nargin = 2;
N = 500;


    reps = 0;
    % pv is a vector containing all of the relevant parameters
    T_b = pv(1);
    s_b = pv(2);
    l_b = pv(3);
    T_x = pv(4);
    s_x = pv(5);
    l_x = pv(6);
    r = pv(7);
    
    randn('seed',12345);
    rand('seed',12345);

    %grids of beta
    gridb=(-2:0.1:2)';
    mb=size(gridb);
    mb=mb(1);
    bone=ones(mb,1);
    mb=mb^2;
    beta=[kron(gridb,bone),kron(bone,gridb),ones(mb,1)]; %grid values of beta, unnormalized
    beta2=sum(beta'.^2)'-1;
    bb=diag(sqrt(beta2+1))\beta;%grid values of beta, normalized- projected to unit sphere

    
 % obtaining s_new, where we store MC results 
    s_new = zeros(sqrt(mb),sqrt(mb));
    
 % MC reps start here 
    while reps < mcreps;
 
         %    
         %DATA GENERATION
         %
            %Random Effect Model-X1=1,Beta3=1, beta is a mixture of two normal
            %distributions

            %generating X
            Xw=[ones(N,1),randn(N,2)];
            Xw2=sqrt(sum(Xw'.^2)');
            X=diag(Xw2)\Xw;
            %generating beta
            
            % Model 1
            muvec = [0,0];
            muvec1 = muvec;
            muvec2 = muvec;
            sigma1=0.3;
            sigma2=0.3;
            rho = 0.0;
                        
            % Model 2
            mu = 0.7;  
            muvec1 = [mu,-mu];
            muvec2 = [-mu,mu];
            sigma1=0.3;
            sigma2=0.3;
            rho=0.5;
                        
            
            SIGMA1=[sigma1,rho*sqrt(sigma1*sigma2);rho*sqrt(sigma1*sigma2),sigma2];
            SIGMA2=[sigma2,rho*sqrt(sigma1*sigma2);rho*sqrt(sigma1*sigma2),sigma1];
            gama1=kron(ones(N,1),muvec1)+randn(N,2)*chol(SIGMA1);
            gama2=kron(ones(N,1),muvec2)+randn(N,2)*chol(SIGMA2);
                        
            indi=(rand(N,1)>=1/2);
            bw=[diag(indi)*gama1+diag(1-indi)*gama2,ones(N,1)];
            
            %generating Y
            Y=zeros(N,1);
            i=1;

            while i<=N;
                Y(i)=(X(i,:)*bw(i,:)'>=0);
                i=i+1;
            end

            fbt1=mvnpdf(beta(:,1:2),muvec1,SIGMA1);
            fbt2=mvnpdf(beta(:,1:2),muvec2,SIGMA2);
            f_original = 0.5*fbt1 + 0.5*fbt2;

            betam1=reshape(beta(:,1),sqrt(mb),sqrt(mb));
            betam2=reshape(beta(:,2),sqrt(mb),sqrt(mb));

            f_new = density(X,Y,bb, T_b, s_b, l_b, T_x, s_x, l_x, r);
            f_new=((beta2+1).^(-3/2)).*f_new;

            f_new=reshape(f_new,sqrt(mb),sqrt(mb));
            s_new = s_new + f_new;
        reps = reps + 1;
        display(reps);
    end;
        
    f_original=reshape(f_original,sqrt(mb),sqrt(mb));
    s_newave = s_new/reps; 
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% f_new is the new estimator, and f_original is the actual density
    
    ssq_new = (f_original - s_newave) .^2;
    
    % plot if specified
    if nargin > 1 && plot == 1
        figure;
        hold on;
        subplot(1,2,1);
        mesh(betam1,betam2,f_original,'edgecolor','black');
        alpha(0.4);
          xlabel(texlabel('beta_1'),'FontSize',10);
        ylabel(texlabel('beta_2'),'FontSize',10);
        zlabel('$f_\beta$','interpreter','latex','FontSize',10);
        subplot(1,2,2);
        mesh(betam1,betam2,s_newave,'edgecolor','black');
        hold on;
        %mesh(betam1,betam2,s_newave,'edgecolor','red');
        alpha(0.4);
        hold off;
        xlabel(texlabel('beta_1'),'FontSize',10);
        ylabel(texlabel('beta_2'),'FontSize',10);
        zlabel('$E[\hat{f_\beta}\: ]$','interpreter','latex','FontSize',10);
    end
      
